#include-once
;
; AutoIt Version: 3.0
; Language:       English
; Platform:       Win9x/NT
; Author:         Michael W. Bayley
;
; Script Function:
;  TrayURL INI File Functions
;
;  High Level Functions
;  --------------------
;  AddCatToIni($title, $cat)
;  RenCatToIni($title, $cat, $newcat)
;  DelCatFromIni($title, $cat, $flag)
;  AddEntryToIni($title, $cat, $name, $url, $login, $paswd, $flags, $date, $flag)
;  EditEntryToIni($title, cat, $name, $newname, $newurl, $newlogin, $newpaswd, $newflags, $newdate, $flag)
;  DelEntryFromIni($title, $cat, $name, $flag)
;
;  Low Level Functions (New)
;  -------------------------
;  CreateRecord($name, $url, $login, $paswd, $flags, $date)
;  ReadRecordByIndex($title, $cat, $index)
;  ReadRecordByName($title, $cat, $name)
;  ReadFieldByIndex($title, $cat, $fieldID, $index)
;  ReadFieldByName($title, $cat, $filedID, $name)
;  WriteRecordByIndex($title, $cat, $index, $record)
;  WriteRecordByName($title, $cat, $name, $data)
;  WriteFieldByIndex($title, $cat, $fieldID, $index, $record)
;  WriteFieldByName($title, $cat, $filedID, $name, $data)
;  WriteEntryByIndex($title, $cat, $index, $name, $url, $login, $paswd, $flags, $date)
;  WriteEntryByName($title, $cat, $findname, $name, $url, $login, $paswd, $flags, $date)
;  DeleteRecordByIndex($title, $cat, $index)
;  DeleteRecordByName($title, $cat, $name)
;
;  Low Level Functions (Old)
;  -------------------------
;  WriteEntryInIni($title, $cat, $index, $name, $url, $login, $paswd, $flags, $date)
;  DeleteEntryFromIni($title, $cat, $index)
;
; Dependancies
;
; <None> 

#include <TrayUrlIniDefs.au3>


;
; User Defined Functions
; INI File High Level Functions
;

;
; AddCatToIni($title, $cat, $flag)
; Entry:       $title      Title text in case of error
;              $cat        Category to add
;              $flag       Report duplicate category flag
; Return:      -1 if catagories are full
;              0 if all ok
;              1 if write failure
; Procedure:   1. Insert category into ini category list.
;              2. Add category empty section.
;              3. Exit all done.
; Assumptions: Assumes that there is category space remaining

Func  AddCatToIni($title, $cat, $flag)
   Local    $items, $index
   Local    $cat2, $cat3
   
   ; 1. Insert category into ini category list.
   $items = Number(IniRead($IniFile, $IniCatSection, $IniItems, 0))
   $cat3 = $cat
   $index = 1
   While $index <= $items
      $cat2 = IniRead($IniFile, $IniCatSection, $IniItem & $index, "")
      If StringLen($cat2) Then
         If $cat == $cat2 Then
            If $flag Then
               MsgBox(0, $title, "Warning - " & $cat2 & " already exists!")
            EndIf
         ElseIf $cat3 < $cat2 Then
            If $items >= $CATMAX Then
               MsgBox(0, $title, "Error - No more categories available!")
               Return(-1)
            EndIf

            If IniWrite($IniFile, $IniCatSection, $IniItem & $index, $cat3) <> 1 Then
               MsgBox(0, $title, "Error - Failure writing category to ini file!")
               Return(1)
            EndIf
            $cat3 = $cat2
         EndIf
      EndIf
      $index = $index + 1
   WEnd
      
   If IniWrite($IniFile, $IniCatSection, $IniItem & $index, $cat3) <> 1 Then
      MsgBox(0, $title, "Error - Failure writing category to ini file!")
      Return(1)
   EndIf
   If IniWrite($IniFile, $IniCatSection, $IniItems,  $index) <> 1 Then
      MsgBox(0, $title, "Error - Failure writing item count to ini file!")
      Return(1)
   EndIf


   ; 2. Add category empty section.
   If IniWrite($IniFile, $cat, $IniItems,  0) <> 1 Then
      MsgBox(0, $title, "Error - Failure writing new category item count to ini file!")
      Return(1)
   EndIf


   ; 3. Exit all done.
   Return(0)
EndFunc


;
; RenCatToIni($title, $cat, $newcat)
; Entry:       $cat        Category to add
;              $newcat     New category name
; Return:      <None>
; Procedure:   1. Remove old category and insert new category into ini category list.
;              2. Rename category section.
;              3. Exit all done.
;

Func  RenCatToIni($title, $cat, $newcat)
   Local    $items, $index, $deleted
   Local    $cat2, $cat3
   
   ; 1. Remove old category and insert new category into ini category list.
   $cat3 = $newcat
   $items = Number(IniRead($IniFile, $IniCatSection, $IniItems, 0))
   $index = 1
   $deleted = 0
   While $index <= $items
      $cat2 = IniRead($IniFile, $IniCatSection, $IniItem & $index, "")
      If StringLen($cat2) Then
         If $cat2 == $cat Then
            $deleted = $deleted + 1
         ElseIf $cat3 < $cat2 Then
            If IniWrite($IniFile, $IniCatSection, $IniItem & ($index - $deleted), $cat3) <> 1 Then
               MsgBox(0, $title, "Error - Failure writing category to ini file!")
            EndIf
            $cat3 = $cat2
         ElseIf $deleted Then
            If IniWrite($IniFile, $IniCatSection, $IniItem & ($index - $deleted), $cat2) <> 1 Then
               MsgBox(0, $title, "Error - Failure writing category to ini file!")
            EndIf
         EndIf
      EndIf
      $index = $index + 1
   WEnd
      
   If IniWrite($IniFile, $IniCatSection, $IniItem & ($index - $deleted), $cat3) <> 1 Then
      MsgBox(0, $title, "Error - Failure writing category to ini file!")
   EndIf


   ; 2. Rename category section.
   If IniRenameSection($IniFile, $cat, $newcat) <> 1 Then
      MsgBox(0, $title, "Error - Failure renaming category in ini file!")
   EndIf


   ; 3. Exit all done.
   Return(0)
EndFunc


;
; DelCatFromIni($title, $cat, $flag)
; Entry:       $cat        Category to delete
; Return:      0 if user aborted
;              1 if successful
; Procedure:   1. Warn if category not empty.
;              2. Delete category from ini category list.
;              3. Delete category section.
;              4. Exit all done.
;

Func  DelCatFromIni($title, $cat, $flag)
   Local    $items, $index, $reply, $deleted
   Local    $cat2

   ; 1. Warn if category not empty.
   $items = Number(IniRead($IniFile, $cat, $IniItems, 0))
   If $Items Then
      $reply = MsgBox(8196, $title, "Warning - Category " & $cat & " is not empty.  Delete anyway?")
      If $reply <> 6 Then
         Return(0)
      EndIf
   ElseIf $flag Then
      $reply = MsgBox(8196, $title, "Are you sure that you want to delete category " & $cat & "?")
      If $reply <> 6 Then
         Return(0)
      EndIf
   EndIf
   

   ; 2. Delete category from ini category list.
   $items = Number(IniRead($IniFile, $IniCatSection, $IniItems, 0))
   $index = 1
   $deleted = 0
   While $index <= $items
      $cat2 = IniRead($IniFile, $IniCatSection, $IniItem & $index, "")
      If $cat == $cat2 Then
         $deleted = $deleted + 1
      ElseIf $deleted Then
        If IniWrite($IniFile, $IniCatSection, $IniItem & ($index - $deleted), $cat2) <> 1 Then
           MsgBox(0, $title, "Error - Failure writing category to ini file!")
         EndIf
      EndIf
      $index = $index + 1
   WEnd
      
   If IniDelete($IniFile, $IniCatSection, $IniItem & ($index - $deleted)) <> 1 Then
      MsgBox(0, $title, "Error - Failure deleting category in category list in ini file!")
   EndIf
   If IniWrite($IniFile, $IniCatSection, $IniItems,  ($index - $deleted - 1)) <> 1 Then
      MsgBox(0, $title, "Error - Failure writing item count to ini file!")
   EndIf


   ; 3. Delete category section.
   If IniDelete($IniFile, $cat) <> 1 Then
      MsgBox(0, $title, "Error - Failure deleting category section from ini file!")
   EndIf


   ; 4. Exit all done.
   Return(1)
EndFunc


;
; AddEntryToIni($title, $cat, $name, $url, $login, $paswd, $flags, $date, $flag)
; Entry:       $title      Dialog title text in case of error
;              $cat        Category to add to
;              name        Name to add
;              $url        URL to add
;              $login      User login to add
;              $paswd      User password to add
;              $flags      User flags to add
;              $date       Visit date to add
;              $flag       Warn if overwrite
; Return:      -1 if category is full
;              0 if user aborted
;              1 if success
; Procedure:   1. Setup new record.
;              2. Warn for overwrite, and overwrite if ok.
;              3. Abort if category is full.
;              4. Insert info into ini list.
;              5. Adjust total.
;              6. Exit all done.
;

Func  AddEntryToIni($title, $cat, $name, $url, $login, $paswd, $flags, $date, $flag)
   Local    $items, $index, $name2
   Local    $record2 = CreateRecord("","","","",0,"")
   Local    $record3 = CreateRecord("","","","",0,"")
   
   ; 1. Setup new record.
   $record3 = CreateRecord($name, $url, $login, $paswd, $flags, $date)


   ; 2.Warn for overwrite, and overwrite if ok.
   $items = Number(IniRead($IniFile, $cat, $IniItems, 0))
   $index = 1
   While $index <= $items
      $name2 = ReadFieldByIndex($title, $cat, $RECNAME, $index)
      If StringLen($name2) Then
         If $name == $name2 Then
            If $flag Then
               $reply = MsgBox(8196, $title, "Warning - " & $name & " already exists.  Overwrite?")
            Else
               $reply = 6
            EndIf
            
            If $reply == 6 Then
               WriteRecordByIndex($title, $cat, $index, $record3)
               Return(1)
            Else
               Return(0)
            EndIf
         ElseIf $name < $name2 Then
            ExitLoop
         EndIf
      EndIf
      $index += 1
   WEnd
   
   
   ; 3. Abort if category is full.
   If $items >= $ITEMMAX Then
       MsgBox(0, $title, "Error - Category " & $cat & " is full!")
       Return(-1)
   EndIf

            
   ; 4. Insert info into ini list.
   While $index <= $items
      $record2 = ReadRecordByIndex($title, $cat, $index)
      WriteRecordByIndex($title, $cat, $index, $record3)
      $record3 = $record2
      $index += 1
   WEnd
   WriteRecordByIndex($title, $cat, $index, $record3)
   

   ; 5. Adjust total.
   If IniWrite($IniFile, $cat, $IniItems, $index) <> 1 Then
      MsgBox(0, $title, "Error - Failure writing new item count for category " & Chr(34) & $cat & Chr(34) & "!")
   EndIf


   ; 6. Exit all done.
   Return(1)
EndFunc


;
; EditEntryToIni($title, cat, $name, $newname, $newurl, $newlogin, $newpaswd, $newflags, $newdate, $flag)
; Entry:       $cat        Category to modify
;              $name       Entry name to replace
;              $newname    New entry name
;              $newurl     New URL
;              $newlogin   New user login
;              $newpaswd   New user password
;              $newflags   New user flags
;              $newdate    New visit date
;              $flag       Overwrite flag
; Return:      0 if user aborted
;              1 if successful
; Procedure:   1. If name did not change then just overwrite it.
;              2. Determine delete index and overwrite or insert index.
;              3. If overwrite then warn, exit if no overwrite.
;              4. If delete index < overwrite index.
;              5. If delete idex > overwrite index.
;              6. If delete index < insert index.
;              7. If delete index > insert index.
;              8. Exit all done.
;

Func  EditEntryToIni($title, $cat, $name, $newname, $newurl, $newlogin, $newpaswd, $newflags, $newdate, $flag)
   Local    $items, $index, $name2
   Local    $delindex, $insindex, $overindex
   Local    $record2 = CreateRecord("", "", "", "", 0, "")
   Local    $record3 = CreateRecord("", "", "", "", 0, "")
   Local    $reply

   ; 1. If name did not change then just overwrite it.
   $items = IniRead($IniFile, $cat, $IniItems, 0)
   $index = 1
   If $name == $newname Then
      $index = WriteEntryByName($title, $cat, $name, $newname, $newurl, $newlogin, $newpaswd, $newflags, $newdate)
      If $index Then Return(1)
      Return(0)
   EndIf


   ; 2. Determine delete index and overwrite or insert index.
   $delindex = 0
   $insindex = 0
   $overindex = 0
   While $index <= $items
      $name2 = ReadFieldByIndex($title, $cat, $RECNAME, $index)
      If $name2 == $name AND $delindex == 0 Then
         $delindex = $index
      ElseIf $newname == $name2 AND $overindex == 0 Then
         $overindex = $index
      ElseIf $newname < $name2 AND $insindex == 0 Then
         $insindex = $index
      EndIf
      
      If $delindex AND ($overindex OR $insindex) Then ExitLoop
      $index += 1
   WEnd
   
   If $delindex == 0 Then
      MsgBox(0, $title, "Error - Can't locate " & Chr(34) & $name & Chr(34) & " in category " & Chr(34) & $cat & Chr(34) & "!")
      Return(0)
   EndIf
   If $insindex == 0 Then $insindex = $items + 1

 
   ; 3. If overwrite then warn, exit if no overwrite.
   If $overindex Then
      If $flag Then
         If MsgBox(260, $title, "Warning - Entry exists!  Overwrite it?") <> 6 Then
            Return(0)
         EndIf
      EndIf


      ; 4. If delete index < overwrite index.
      If $delindex < $overindex Then
         $index = $delindex + 1
         While $index < $overindex
            $record2 = ReadRecordByIndex($title, $cat, $index)
            WriteRecordByIndex($title, $cat, $index - 1)
            $index += 1
         WEnd
         WriteEntryByIndex($title, $cat, $index - 1, $newname, $newurl, $newlogin, $newpaswd, $newflags, $newdate)
         $index += 1
         
         While $index <= $items
            $record2 = ReadRecordByIndex($title, $cat, $index)
            WriteRecordByIndex($title, $cat, $index - 1, $record2)
            $index += 1
         WEnd
         DeleteRecordByIndex($title, $cat, $index - 1)
         If IniWrite($IniFile, $cat, $IniItems, $index - 2) <> 1 Then
            MsgBox(0, $title, "Error - Error writing new item count in category " & Chr(34) & $cat & Chr(34) & "!")
         EndIf
      Else
         ; 5. If delete idex > overwrite index.
         WriteEntryByIndex($title, $cat, $overindex, $newname, $newurl, $newlogin, $newpaswd, $newflags, $newdate)
         $index += 1

         $index = $delindex + 1         
         While $index <= $items
            $record2 = ReadRecordByIndex($title, $cat, $index)
            WriteRecordByIndex($title, $cat, $index - 1, $record2)
            $index += 1
         WEnd

         DeleteRecordByIndex($title, $cat, $index - 1)
         If IniWrite($IniFile, $cat, $IniItems, $index - 2) <> 1 Then
            MsgBox(0, $title, "Error - Error writing new item count in category " & Chr(34) & $cat & Chr(34) & "!")
         EndIf
      EndIf
   Else
      ; 6. If delete index < insert index.
      If $delindex < $insindex Then
         $index = $delindex + 1
         While $index < $insindex
            $record2 = ReadRecordByIndex($title, $cat, $index)
            WriteRecordByIndex($title, $cat, $index - 1, $record2)
            $index += 1
         WEnd
         WriteEntryByIndex($title, $cat, $index - 1, $newname, $newurl, $newlogin, $newpaswd, $newflags, $newdate)
      Else
         ; 7. If delete index > insert index.
         $record3 = CreateRecord($newname, $newurl, $newlogin, $newpaswd, $newflags, $newdate)

         $index = $insindex
         While $index < $delindex
            $record2 = ReadRecordByIndex($title, $cat, $index)
            WriteRecordByIndex($title, $cat, $index, $record3)
            $record3 = $record2
            $index += 1
         WEnd
         WriteRecordByIndex($title, $cat, $index, $record3)
      EndIf
   EndIf


   ; 8. Exit all done.
   Return(1)
EndFunc


;
; DelEntryFromIni($title, $cat, $name, $flag)
; Entry:       $cat        Category to delete from
;              $name       Name to delete
;              $flag       Warn user
; Return:      0 if user aborted
;              1 if successful
; Procedure:   1. Warn if warning is active.
;              2. Find entry to delete.
;              3. Error if not found.
;              4. Delete entry from list.
;              5. Update count.
;              6. Exit all done.
;

Func  DelEntryFromIni($title, $cat, $name, $flag)
   Local    $items, $index, $reply, $deleted
   Local    $record2 = CreateRecord("", "", "", "", 0, "")

   ; 1. Warn if warning is active.
   If $flag Then
      $reply = MsgBox(8196, $title, "Are you sure that you want to delete item " & $name & "?")
      If $reply <> 6 Then
         Return(0)
      EndIf
   EndIf


   ; 2. Find entry to delete.
   $items = Number(IniRead($IniFile, $cat, $IniItems, 0))
   $index = 1
   While $index <= $items
      $name2 = ReadFieldByIndex($title, $cat, $RECNAME, $index)
      If $name2 == $name Then
         ExitLoop
      EndIf
     $index += 1
   WEnd


   ; 3. Error if not found.
   If $index > $items Then
      MsgBox(0, $title, "Error - Can't locate item " & Chr(34) & $name & Chr(34) & " in category " & Chr(34) & $cat & Chr(34) & "!")
      Return(1)
   EndIf
   

   ; 4. Delete entry from list.
   $index += 1
   While $index <= $items
      $record2 = ReadRecordByIndex($title, $cat, $index)
      WriteRecordByIndex($title, $cat, $index - 1, $record2)
      $index += 1
   WEnd

   DeleteRecordByIndex($title, $cat, $index - 1)
   

   ; 5. Update count.
   If IniWrite($IniFile, $cat, $IniItems,  ($index - 2)) <> 1 Then
      MsgBox(0, $title, "Error - Failure writing item count to category " & Chr(34) & $cat & Chr(34) & "!")
   EndIf


   ; 6. Exit all done.
   Return(1)
EndFunc


;
; User Defined Functions
; INI File Low Level Functions (New)
;

;
; CreateRecord($name, $url, $login, $paswd, $flags, $date)
; Entry:       $name       Name to use.
;              $url        URL to use.
;              $login      Login to use.
;              $paswd      Password to use.
;              $flags      Flags to use.
;              $date       Date to use.
; Return:      $record     New record
; Procedure:   1. Build the record.
;              2. Return the record.
;
Func  CreateRecord($name, $url, $login, $paswd, $flags, $date)
   Local    $record, $item

   ; 1. Build the record.
   $item = $name
   $item &= $RECSEP & $url
   $item &= $RECSEP & $login
   $item &= $RECSEP & $paswd
   $item &= $RECSEP & $flags
   $item &= $RECSEP & $date

   $record = StringSplit($item, $RECSEP)


   ; 2. Return the record.
   Return($record)
EndFunc


;
; ReadRecordByIndex($title, $cat, $index)
; Entry:       $title      Dialog title in event of error
;              $cat        Category to read from
;              $index      Item index within categoty
; Return:      $record     The record that was read.
;              @error = 1  If fail
;              @error = 0  If ok
; Procedure:   1. Read the record.
;              2. Split the record into fields.
;              3. Return the data.
;
Func  ReadRecordByIndex($title, $cat, $index)
   Local    $record, $item

   ; 1. Read the record.
   $item = IniRead($IniFile, $cat, $IniItem & $index, "")


   ; 2. Split the record into fields.
   $record = StringSplit($item, $RECSEP)
   If $record[$RECCOUNT] <> 6 Then
      MsgBox(0, $title, "Error - Failure reading record " & $index & " to category " & Chr(34) & $cat & Chr(34) & "!")
      SetError(1)
   EndIf


   ; 3. Return the data.
   Return($record)
EndFunc


;
; ReadRecordByName($title, $cat, $name)
; Entry:       $title      Dialog title in event of error
;              $cat        Category to read from
;              $name       Name contained within categoty
; Return:      $record     Record that was read
;              @error = 1  If fail
;              @error = 0  If ok
; Procedure:   1. Get record count.
;              2. Find the record, error if not found.
;              3. Return the data.
;
Func  ReadRecordByName($title, $cat, $name)
   Local    $record, $items, $index, $item

   ; 1. Get record count.
   $items = Number(IniRead($IniFile, $cat, $IniItems, 0))


   ; 2. Find the record, error if not found.
   $index = 1
   While $index <= $items
      $item = IniRead($IniFile, $cat, $IniItem & $index, "")
      $record = StringSplit($item, $RECSEP)

      If $record[$RECNAME] == $name Then ExitLoop
      $index += 1
   WEnd
   

   ; 3. Return the data.
   If $record[$RECNAME] == $name Then Return($record)

   MsgBox(0, $title, "Error - Failure locating item " & Chr(34) & $name & Chr(34) & " in category " & Chr(34) & $cat & Chr(34) & "!")
   SetError(1)
   Return($record)
EndFunc


;
; ReadFieldByIndex($title, $cat, $fieldID, $index)
; Entry:       $title      Dialog title in event of error
;              $cat        Category to read from
;              $fieldID    ID of field to return
;              $index      Item index within categoty
; Return:      %data       Field content
;              @error = 1  If fail
;              @error = 0  If ok
; Procedure:   1. Read the record.
;              2. Split the record into fields.
;              3. Return the data.
;
Func  ReadFieldByIndex($title, $cat, $fieldID, $index)
   Local    $record, $item

   ; 1. Read the record.
   $item = IniRead($IniFile, $cat, $IniItem & $index, "")


   ; 2. Split the record into fields.
   $record = StringSplit($item, $RECSEP)
   If $record[$RECCOUNT] <> 6 Then
      MsgBox(0, $title, "Error - Failure reading record " & $index & " to category " & Chr(34) & $cat & Chr(34) & "!")
      SetError(1)
      Return("")
   EndIf


   ; 3. Return the data.
   Return($record[$fieldID])
EndFunc


;
; ReadFieldByName($title, $cat, $filedID, $name)
; Entry:       $title      Dialog title in event of error
;              $cat        Category to read from
;              $fieldID    ID of field to return
;              $name       Name contained within categoty
; Return:      $data       Field data
;              @error = 1 if name not found
;              @error = 0 if all ok
; Procedure:   1. Get record count.
;              2. Find the record, error if not found.
;              3. Return the data.
;
Func  ReadFieldByName($title, $cat, $fieldID, $name)
   Local    $record, $items, $index, $item

   ; 1. Get record count.
   $items = Number(IniRead($IniFile, $cat, $IniItems, 0))


   ; 2. Find the record, error if not found.
   $index = 1
   While $index <= $items
      $item = IniRead($IniFile, $cat, $IniItem & $index, "")
      $record = StringSplit($item, $RECSEP)

      If $record[$RECNAME] == $name Then ExitLoop
      $index += 1
   WEnd


   ; 3. Return the data.
   If $record[$RECNAME] == $name Then Return($record[$fieldID])

   MsgBox(0, $title, "Error - Failure locating item " & Chr(34) & $name & Chr(34) & " in category " & Chr(34) & $cat & Chr(34) & "!")
   SetError(1)
   Return("")
EndFunc


;
; WriteRecordByIndex($title, $cat, $index, $record)
; Entry:       $title      Dialog title in event of error
;              $cat        Category to read from
;              $index      Item index within categoty
;              $record     Record to write.
; Return:      0 if fail
;              Index of record if successful
; Procedure:   1. Build the record.
;              2. Write the data, report if error.
;              3. Exit all done.
;
Func  WriteRecordByIndex($title, $cat, $index, $record)
   Local    $item

   ; 1. Build the record.
   $item = $record[$RECNAME]
   $item &= $RECSEP & $record[$RECURL]
   $item &= $RECSEP & $record[$RECLOGIN]
   $item &= $RECSEP & $record[$RECPASWD]
   $item &= $RECSEP & $record[$RECFLAGS]
   $item &= $RECSEP & $record[$RECDATE]


   ; 2. Write the data, report if error.
   If IniWrite($IniFile, $cat, $IniItem & $index, $item) <> 1 Then
      MsgBox(0, $title, "Error - Failure writing record " & $index & " to category " & Chr(34) & $cat & Chr(34) & "!")
      Return(0)
   EndIf


   ; 3. Exit all done.
   Return($index)
EndFunc


;
; WriteRecordByName($title, $cat, $name, $record)
; Entry:       $title      Dialog title in event of error
;              $cat        Category to read from
;              $name       Name contained within categoty
;              $record     Record to write
; Return:      0 if fail
;              Index of record if successful
; Procedure:   1. Get record count.
;              2. Find the record, error if not found.
;              3. Build the record.
;              4. Write the data, report if error.
;              5. Exit all done.
;
Func  WriteRecordByName($title, $cat, $name, $record)
   Local    $record2, $items, $index, $item

   ; 1. Get record count.
   $items = Number(IniRead($IniFile, $cat, $IniItems, 0))


   ; 2. Find the record, error if not found.
   $index = 1
   While $index <= $items
      $item = IniRead($IniFile, $cat, $IniItem & $index, "")
      $record2 = StringSplit($item, $RECSEP)

      If $record2[$RECNAME] == $name Then ExitLoop
      $index += 1
   WEnd

   If $index > $items Then
      MsgBox(0, $title, "Error - Failure locating item " & Chr(34) & $name & Chr(34) & " in category " & Chr(34) & $cat & Chr(34) & "!")
      Return(0)
   EndIf


   ; 3. Build the record.
   $item = $record[$RECNAME]
   $item &= $RECSEP & $record[$RECURL]
   $item &= $RECSEP & $record[$RECLOGIN]
   $item &= $RECSEP & $record[$RECPASWD]
   $item &= $RECSEP & $record[$RECFLAGS]
   $item &= $RECSEP & $record[$RECDATE]


   ; 4. Write the data, report if error.
   If IniWrite($IniFile, $cat, $IniItem & $index, $item) <> 1 Then
      MsgBox(0, $title, "Error - Failure writing record " & $index & " to category " & Chr(34) & $cat & Chr(34) & "!")
      Return(0)
   EndIf


   ; 5. Exit all done.
   Return($index)
EndFunc


;
; WriteFieldByIndex($title, $cat, $fieldID, $index, $data)
; Entry:       $title      Dialog title in event of error
;              $cat        Category to read from
;              $fieldID    Field ID number to write
;              $index      Index of category to write
;              $data       Field data to write
; Return:      0 if fail
;              Index of record if successful
; Procedure:   1. Read the record.
;              2. Build the record.
;              3. Write the data, report if error.
;              4. Exit all done.
;
Func  WriteFieldByIndex($title, $cat, $fieldID, $index, $data)
   Local    $record, $item

   ; 1. Read the record.
   $item = IniRead($IniFile, $cat, $IniItem & $index, "")
   If StringLen($item) == 0 Then
      MsgBox(0, $title, "Error - Failure reading record " & $index & " to category " & Chr(34) & $cat & Chr(34) & "!")
      Return(0)
   EndIf


   ; 2. Build the record.
   $record = StringSplit($item, $RECSEP)
   $record[$fieldID] = $data

   $item = $record[$RECNAME]
   $item &= $RECSEP & $record[$RECURL]
   $item &= $RECSEP & $record[$RECLOGIN]
   $item &= $RECSEP & $record[$RECPASWD]
   $item &= $RECSEP & $record[$RECFLAGS]
   $item &= $RECSEP & $record[$RECDATE]


   ; 3. Write the data, report if error.
   If IniWrite($IniFile, $cat, $IniItem & $index, $item) <> 1 Then
      MsgBox(0, $title, "Error - Failure writing record " & $index & " to category " & Chr(34) & $cat & Chr(34) & "!")
      Return(0)
   EndIf


   ; 4. Exit all done.
   Return($index)
EndFunc


;
; WriteFieldByName($title, $cat, $filedID, $name, $data)
; Entry:       $title      Dialog title in event of error
;              $cat        Category to read from
;              $fieldID    Field ID of the record
;              $name       Name contained within categoty
;              $data       Field data to write
; Return:      0 if fail
;              Index of record if successful
; Procedure:   1. Get record count.
;              2. Find the record, error if not found.
;              3. Build the record.
;              4. Write the data, report if error.
;              5. Exit all done.
;
Func  WriteFieldByName($title, $cat, $fieldID, $name, $data)
   Local    $record, $items, $index, $item

   ; 1. Get record count.
   $items = Number(IniRead($IniFile, $cat, $IniItems, 0))


   ; 2. Find the record, error if not found.
   $index = 1
   While $index <= $items
      $item = IniRead($IniFile, $cat, $IniItem & $index, "")
      $record = StringSplit($item, $RECSEP)

      If $record[$RECNAME] == $name Then ExitLoop
      $index += 1
   WEnd

   If $index > $items Then
      MsgBox(0, $title, "Error - Failure locating item " & Chr(34) & $name & Chr(34) & " in category " & Chr(34) & $cat & Chr(34) & "!")
      Return(0)
   EndIf


   ; 3. Build the record.
   $record[$fieldID] = $data

   $item = $record[$RECNAME]
   $item &= $RECSEP & $record[$RECURL]
   $item &= $RECSEP & $record[$RECLOGIN]
   $item &= $RECSEP & $record[$RECPASWD]
   $item &= $RECSEP & $record[$RECFLAGS]
   $item &= $RECSEP & $record[$RECDATE]


   ; 4. Write the data, report if error.
   If IniWrite($IniFile, $cat, $IniItem & $index, $item) <> 1 Then
      MsgBox(0, $title, "Error - Failure writing record named " & Chr(34) & $name & Chr(34) & " to category " & Chr(34) & $cat & Chr(34) & "!")
      Return(0)
   EndIf


   ; 5. Exit all done.
   Return($index)
EndFunc


;
; WriteEntryByIndex($title, $cat, $index, $name, $url, $login, $paswd, $flags, $date)
; Entry:       $title      Dialog title in event of error
;              $cat        Category to read from
;              $index      Item index within categoty
;              $name       Name to write.
;              $url        URL to write.
;              $login      Login to write.
;              $paswd      Password to write.
;              $flags      Flags to write.
;              $date       Date to write.
; Return:      0 if fail
;              Index of record if successful
; Procedure:   1. Build the record.
;              2. Write the data, report if error.
;              3. Exit all done.
;
Func  WriteEntryByIndex($title, $cat, $index, $name, $url, $login, $paswd, $flags, $date)
   Local    $item

   ; 1. Build the record.
   $item = $name
   $item &= $RECSEP & $url
   $item &= $RECSEP & $login
   $item &= $RECSEP & $paswd
   $item &= $RECSEP & $flags
   $item &= $RECSEP & $date


   ; 2. Write the data, report if error.
   If IniWrite($IniFile, $cat, $IniItem & $index, $item) <> 1 Then
      MsgBox(0, $title, "Error - Failure writing record " & $index & " to category " & Chr(34) & $cat & Chr(34) & "!")
      Return(0)
   EndIf


   ; 3. Exit all done.
   Return($index)
EndFunc


;
; WriteEntryByName($title, $cat, $findname, $name, $url, $login, $paswd, $flags, $date)
; Entry:       $title      Dialog title in event of error
;              $cat        Category to read from
;              $name       Name contained within categoty
;              $url        URL to write.
;              $login      Login to write.
;              $paswd      Password to write.
;              $flags      Flags to write.
;              $date       Date to write.
; Return:      0 if fail
;              Index of record if successful
; Procedure:   1. Get record count.
;              2. Find the record, error if not found.
;              3. Build the record.
;              4. Write the data, report if error.
;              5. Exit all done.
;
Func  WriteEntryByName($title, $cat, $findname, $name, $url, $login, $paswd, $flags, $date)
   Local    $record, $items, $index, $item

   ; 1. Get record count.
   $items = Number(IniRead($IniFile, $cat, $IniItems, 0))


   ; 2. Find the record, error if not found.
   $index = 1
   While $index <= $items
      $item = IniRead($IniFile, $cat, $IniItem & $index, "")
      $record = StringSplit($item, $RECSEP)

      If $record[$RECNAME] == $findname Then ExitLoop
      $index += 1
   WEnd

   If $index > $items Then
      MsgBox(0, $title, "Error - Failure locating item " & Chr(34) & $name & Chr(34) & " in category " & Chr(34) & $cat & Chr(34) & "!")
      Return(0)
   EndIf


   ; 3. Build the record.
   $item = $name
   $item &= $RECSEP & $url
   $item &= $RECSEP & $login
   $item &= $RECSEP & $paswd
   $item &= $RECSEP & $flags
   $item &= $RECSEP & $date


   ; 4. Write the data, report if error.
   If IniWrite($IniFile, $cat, $IniItem & $index, $item) <> 1 Then
      MsgBox(0, $title, "Error - Failure writing record named " & Chr(34) & $name & Chr(34) & " to category " & Chr(34) & $cat & Chr(34) & "!")
      Return(0)
   EndIf


   ; 5. Exit all done.
   Return($index)
EndFunc


;
; DeleteRecordByIndex($title, $cat, $index)
; Entry:       $title      Dialog title in event of error
;              $cat        Category to delete
;              $index      Item index within categoty
; Return:      0 if fail
;              Index of record if successful
; Procedure:   1. Delete the record, report if error.
;              2. Exit all done.
;
Func  DeleteRecordByIndex($title, $cat, $index)
   ; 1. Delete the record, report if error.
   If IniDelete($IniFile, $cat, $IniItem & $index) <> 1 Then
      MsgBox(0, $title, "Error - Failure deleting record " & $index & " from category " & Chr(34) & $cat & Chr(34) & "!")
      Return(0)
   EndIf


   ; 2. Exit all done.
   Return($index)
EndFunc


;
; DeleteRecordByName($title, $cat, $name)
; Entry:       $title      Dialog title in event of error
;              $cat        Category to delete
;              $name       Name contained within categoty
; Return:      0 if fail
;              Index of record if successful
; Procedure:   1. Get record count.
;              2. Find the record, error if not found.
;              3. Delete the data, report if error.
;              4. Exit all done.
;
Func  DeleteRecordByName($title, $cat, $name)
   Local    $record, $items, $index, $item

   ; 1. Get record count.
   $items = Number(IniRead($IniFile, $cat, $IniItems, 0))


   ; 2. Find the record, error if not found.
   $index = 1
   While $index <= $items
      $item = IniRead($IniFile, $cat, $IniItem & $index, "")
      $record = StringSplit($item, $RECSEP)

      If $record[$RECNAME] == $name Then ExitLoop
      $index += 1
   WEnd

   If $index > $items Then
      MsgBox(0, $title, "Error - Failure locating item " & Chr(34) & $name & Chr(34) & " in category " & Chr(34) & $cat & Chr(34) & "!")
      Return(0)
   EndIf


   ; 3. Delete the data, report if error.
   If IniDelete($IniFile, $cat, $IniItem & $index) <> 1 Then
      MsgBox(0, $title, "Error - Failure deleting record named " & Chr(34) & $name & Chr(34) & " from category " & Chr(34) & $cat & Chr(34) & "!")
      Return(0)
   EndIf


   ; 4. Exit all done.
   Return($index)
EndFunc


;
; User Defined Functions
; INI File Low Level Functions (Old)
;

;
; WriteEntryToIni($title, $cat, $index, $name, $url, $login, $paswd, $flags, $date)
; Entry:       $title      Dialog title in event of error
;              $cat        Category to delete from
;              $index      Item index within categoty
;              $name       Name to write
;                          Skip if NULL
;              $url        URL to write
;              $login      User login to write
;              $flags      User flags to write
;              $date       Last visit date to write
; Return:      0 if fail
;              1 if successful
; Procedure:   1. Write name if not blank, report if error.
;              2. Write URL, report if error.
;              3. Write login, report if error.
;              4. Write password, report if error.
;              5. Write flags, report if error.
;              6. Write date, report if error.
;              7. Exit all done.
;

Func  WriteEntryToIni($title, $cat, $index, $name, $url, $login, $paswd, $flags, $date)
   ; 1. Write name if not blank, report if error.
   If StringLen($name) Then
      If IniWrite($IniFile, $cat, $IniName & $index, $name) <> 1 Then
         MsgBox(0, $title, "Error - Failure writing description in category " & Chr(34) & $cat & Crr(34) " at index " & $index & "!")
      EndIf
   EndIf

   ; 2. Write URL, report if error.
   If IniWrite($IniFile, $cat, $IniURL & $index, $url) <> 1 Then
      MsgBox(0, $title, "Error - Failure writing URL in category " & Chr(34) & $cat & Crr(34) " at index " & $index & "!")
   EndIf

   ; 3. Write login, report if error.
   If IniWrite($IniFile, $cat, $IniLogin & $index, $login) <> 1 Then
      MsgBox(0, $title, "Error - Failure writing login in category " & Chr(34) & $cat & Crr(34) " at index " & $index & "!")
   EndIf

   ; 4. Write password, report if error.
   If IniWrite($IniFile, $cat, $IniPaswd & $index, $paswd) <> 1 Then
      MsgBox(0, $title, "Error - Failure writing password in category " & Chr(34) & $cat & Crr(34) " at index " & $index & "!")
   EndIf

   ; 5. Write flags, report if error.
   If IniWrite($IniFile, $cat, $IniFlags & $index, $flags) <> 1 Then
      MsgBox(0, $title, "Error - Failure writing flags in category " & Chr(34) & $cat & Crr(34) " at index " & $index & "!")
   EndIf

   ; 6. Write date, report if error.
   If IniWrite($IniFile, $cat, $IniDate & $index, $date) <> 1 Then
      MsgBox(0, $title, "Error - Failure writing date in category " & Chr(34) & $cat & Crr(34) " at index " & $index & "!")
   EndIf

   ; 7. Exit all done.
   Return(1)
EndFunc


;
; DeleteEntryFromIni($title, $cat, $index)
; Entry:       $title      Dialog title in event of error
;              $cat        Category to delete from
;              $index      Item index within categoty
; Return:      0 if fail
;              1 if successful
; Procedure:   1. Delete name, report if error.
;              2. Delete URL, report if error.
;              3. Delete login, report if error.
;              4. Delete password, report if error.
;              5. Delete flags, report if error.
;              6. Delete date, report if error.
;              7. Exit all done.
;

Func  DeleteEntryFromIni($title, $cat, $index)
   ; 1. Delete name, report if error.
   If IniDelete($IniFile, $cat, $IniName & $index) <> 1 Then
      MsgBox(0, $title, "Error - Failure deleting description in category " & Chr(34) & $cat & Crr(34) " at index " & $index & "!")
   EndIf

   ; 2. Write URL, report if error.
   If IniDelete($IniFile, $cat, $IniURL & $index) <> 1 Then
      MsgBox(0, $title, "Error - Failure deleting URL in category " & Chr(34) & $cat & Crr(34) " at index " & $index & "!")
   EndIf

   ; 3. Write login, report if error.
   If IniDelete($IniFile, $cat, $IniLogin & $index) <> 1 Then
      MsgBox(0, $title, "Error - Failure deleting login in category " & Chr(34) & $cat & Crr(34) " at index " & $index & "!")
   EndIf

   ; 4. Write password, report if error.
   If IniDelete($IniFile, $cat, $IniPaswd & $index) <> 1 Then
      MsgBox(0, $title, "Error - Failure deleting password in category " & Chr(34) & $cat & Crr(34) " at index " & $index & "!")
   EndIf

   ; 5. Delete flags, report if error.
   If IniDelete($IniFile, $cat, $IniFlags & $index) <> 1 Then
      MsgBox(0, $title, "Error - Failure deleting flags in category " & Chr(34) & $cat & Crr(34) " at index " & $index & "!")
   EndIf

   ; 6. Delete date, report if error.
   If IniDelete($IniFile, $cat, $IniDate & $index) <> 1 Then
      MsgBox(0, $title, "Error - Failure deleting date in category " & Chr(34) & $cat & Crr(34) " at index " & $index & "!")
   EndIf

   ; 7. Exit all done.
   Return(1)
EndFunc

